@@ -15,7 +15,7 @@ class ModelView(View):
     # BASE
     model_name = None
     public_index = 'index'
-    private_index = 'own'
+    private_index = 'index'
     perpage = 20
     filterform = None
     sortform = None
@@ -26,11 +26,17 @@ class ModelView(View):
 
     # TEMPLATES
     templates = {
+        # Custom by default
+        'index': '{model}/index.html',
+        'card': '{model}/card.html',
+        'create': '{model}/form.html',
+        'edit': '{model}/form.html',
+        'detail': '{model}/detail.html',
+        'pdf': '{model}/pdf.html',
+        'txt': '{model}/txt.html',
+
+        # General by default
         'index_or_detail': 'model/index-or-detail.html',
-        'index': 'model/index.html',
-        'own': 'model/index.html',
-        'favs': 'model/index.html',
-        'search': 'model/search.html',
         'tab_filters': 'model/tab.filters.html',
         'tab_sorts': 'model/tab.sorts.html',
         'tab_perpage': 'model/tab.perpage.html',
@@ -38,12 +44,6 @@ class ModelView(View):
         'filters': 'model/filters.html',
         'loop': 'model/loop.html',
         'pagination': 'model/pagination.html',
-        'card': '{model}/card.html',
-        'create': '{model}/form.html',
-        'edit': '{model}/form.html',
-        'detail': '{model}/detail.html',
-        'pdf': '{model}/pdf.html',
-        'txt': '{model}/txt.html',
     }
 
 
@@ -66,13 +66,6 @@ class ModelView(View):
 
 
     # ROUTES -------------------------------------------------------------------
-    @route('/search/', endpoint='{view}:search', methods=('GET', 'POST'))
-    @route('/search.json', endpoint='{view}:search_json', methods=('GET', 'POST'))
-    @route('/search.html', endpoint='{view}:search_html', methods=('GET', 'POST'))
-    def search(self):
-        return self._index('search')
-
-
     @route('/', endpoint='{view}:index', methods=('GET', 'POST'))
     @route('/index.json', endpoint='{view}:index_json', methods=('GET', 'POST'))
     @route('/index.html', endpoint='{view}:index_html', methods=('GET', 'POST'))
@@ -80,20 +73,6 @@ class ModelView(View):
         return self._index('index')
 
 
-    @route('/own/', endpoint='{view}:own', methods=('GET', 'POST'))
-    @route('/own/index.json', endpoint='{view}:own_json', methods=('GET', 'POST'))
-    @route('/own/index.html', endpoint='{view}:own_html', methods=('GET', 'POST'))
-    def own(self):
-        return self._index('own')
-
-
-    @route('/favs/', endpoint='{view}:favs', methods=('GET', 'POST'))
-    @route('/favs/index.json', endpoint='{view}:favs_json', methods=('GET', 'POST'))
-    @route('/favs/index.html', endpoint='{view}:favs_html', methods=('GET', 'POST'))
-    def favs(self):
-        return self._index('favs')
-
-
     @route('/<id:id>/detail', endpoint='{view}:detail')
     @route('/<id:id>/detail.json', endpoint='{view}:detail_json')
     @route('/<id:id>/detail.pdf', endpoint='{view}:detail_pdf')
@@ -144,7 +123,7 @@ class ModelView(View):
         paginator, redirect = self.do_paginate(cursor, index)
         if redirect:
             return redirect
-        data = self.do_index(paginator, index)
+        data = self.after_paginate(paginator, index)
 
         try:
             if request.path.endswith('.json'):
@@ -173,7 +152,7 @@ class ModelView(View):
                     redirect = None,
                 )
             else:
-                return self.render(index,
+                return self.render('index',
                     index = index,
                     filterform = filterform,
                     sortform = sortform,
@@ -274,7 +253,7 @@ class ModelView(View):
         self.do_delete([model])
         flash_html = render_template('alerts.html')
         redirect = request.args.get('redirect')
-        return jsonify(
+        return self.jsonify(
             data_html = '',
             flash_html = flash_html,
             redirect = redirect,
@@ -289,7 +268,7 @@ class ModelView(View):
         self.do_archive([model])
         flash_html = render_template('alerts.html')
         redirect = request.args.get('redirect')
-        return jsonify(
+        return self.jsonify(
             data_html = '',
             flash_html = flash_html,
             redirect = redirect,
@@ -327,8 +306,6 @@ class ModelView(View):
             filters = {}
             form = None
 
-        self.force_filters(index, filters)
-
         return filters, form
 
 
@@ -412,8 +389,12 @@ class ModelView(View):
         return paginator, None
 
 
-    def do_index(self, paginator, index):
-        raise Exception('Define `{}.do_index` method'.format(self.__class__.__name__))
+    def after_paginate(self, paginator, index):
+        return {}
+
+
+    def detail_model(self, id):
+        raise Exception('Define `{}.detail_model` method'.format(self.__class__.__name__))
 
 
     def create_model(self):
@@ -424,15 +405,11 @@ class ModelView(View):
         raise Exception('Define `{}.edit_model` method'.format(self.__class__.__name__))
 
 
-    def detail_model(self, id):
-        raise Exception('Define `{}.detail_model` method'.format(self.__class__.__name__))
-
-
     def do_create(self, model, form):
         raise Exception('Define `{}.do_create` method'.format(self.__class__.__name__))
 
 
-    def do_edit(self, model):
+    def do_edit(self, model, form):
         raise Exception('Define `{}.do_edit` method'.format(self.__class__.__name__))
 
 
@@ -446,18 +423,21 @@ class ModelView(View):
 
     # HELPERS ------------------------------------------------------------------
     def allows(self, action, models=[], with_filters=True):
-        return g.user.allowed(
-            '{}.{}'.format(self.endpoint, action), models, with_filters=with_filters
-        )
+        if hasattr(self, action) and getattr(self, action) and getattr(self, action).routable:
+            return g.user.allowed(
+                '{}.{}'.format(self.endpoint, action), models, with_filters=with_filters
+            )
+        else:
+            return False
 
 
     def autoredirect(self, model_id):
-        _continue = self.session.get('continue', 'edit')
-        if _continue == 'edit':
+        continue_ = request.form.get('continue') or request.args.get('continue') or self.session.get('continue') or 'edit'
+        if continue_ == 'edit':
             return self.redirect('edit', id=model_id)
-        elif _continue == 'create':
+        elif continue_ == 'create':
             return self.redirect('create')
-        elif _continue == 'index':
+        elif continue_ == 'index':
             index = self.private_index or self.public_index or 'home'
             return self.redirect(index)
         else:
